const FORMAT_PARTS = 12345.6;
const FORMAT_NUMERALS = 9876543210;
/**
 * @public
 */
export class NumberParser {
    constructor(locale) {
        const parts = new Intl.NumberFormat(locale).formatToParts(FORMAT_PARTS);
        const numerals = [
            ...new Intl.NumberFormat(locale, { useGrouping: false }).format(FORMAT_NUMERALS),
        ].reverse();
        const index = new Map(numerals.map((d, i) => [d, i]));
        this._group = new RegExp(`[${parts.find((d) => d.type === 'group').value}]`, 'g');
        this._decimal = new RegExp(`[${parts.find((d) => d.type === 'decimal').value}]`);
        this._separator = new RegExp(`[$${parts.find((d) => d.type === 'decimal').value}]$`);
        this._numeral = new RegExp(`[${numerals.join('')}]`, 'g');
        this._index = (d) => index.get(d);
    }
    parse(localeNumber) {
        return (localeNumber = localeNumber
            .trim()
            .replace(this._group, '')
            .replace(this._decimal, '.')
            .replace(this._numeral, this._index))
            ? +localeNumber
            : NaN;
    }
    hasSeparator(localeNumber) {
        return this._separator.test(localeNumber);
    }
}
