import { __classPrivateFieldGet, __classPrivateFieldSet } from "tslib";
/**
 * @beta
 * Mixin class to expose `shouldRunConnect` and `shouldRunDisconnect`
 *
 * @remarks These can be used to control what lifecycle functionality is ran when the elements
 * are part of a custom layout. The class itself handles all events
 * @privateRemarks
 * #_ syntax is used for a javascript private method/variable
 * we can't use private/protected from typescript in an anonymous class like this
 */
export const LifecycleMixin = (Base) => { var _instances, __container, __tryFindContainingLayout, _a; return _a = class extends Base {
        constructor(...args) {
            super(args);
            _instances.add(this);
            /** @internal **/
            __container.set(this, { _key: 'dom' });
        }
        /**
         * @privateRemarks
         * Rather than using the basic implementation of cloning we run a copy constructor (deepClone)
         * and then manually clone the children too so they can intercept with an overriden cloneNode
         * if they want to
         */
        cloneNode(deep) {
            const thisClone = this.deepClone();
            if (deep) {
                Array.from(this.childNodes).forEach((child) => {
                    thisClone.appendChild(child.cloneNode(true));
                });
            }
            return thisClone;
        }
        /**
         * @privateRemarks
         * Basic implementation of a copy constructor which creates a new element of the same class
         * and copies all attributes over.
         * None-trivial elements will likely want to override this function, call it using super.deepClone(),
         * and then perform extra setup such as copying properties, setting up event listeners
         */
        deepClone() {
            const copy = document.createElement(this.tagName.toLowerCase());
            this.getAttributeNames().forEach((at) => copy.setAttribute(at, this.getAttribute(at)));
            return copy;
        }
        /**
         * @beta
         * @returns - boolean controlling whether to run all disconnectedCallback lifecycle functionality
         */
        get shouldRunDisconnect() {
            if (__classPrivateFieldGet(this, __container, "f")._key === 'dom') {
                return true;
            }
            if (__classPrivateFieldGet(this, __container, "f")._key === 'foundation-layout-cache') {
                return false;
            }
            // _key === 'foundation-layout'
            return !(__classPrivateFieldGet(this, __container, "f").dragging || !__classPrivateFieldGet(this, __container, "f").hasFirstLoaded);
        }
        /**
         * @beta
         * @returns - boolean controlling whether to run all connectedCallback lifecycle functionality
         * @privateRemarks
         * For now its just the same logic as shouldRunDisconnect
         */
        get shouldRunConnect() {
            return this.shouldRunDisconnect;
        }
        connectedCallback() {
            super.connectedCallback();
            if (__classPrivateFieldGet(this, __container, "f")._key !== 'foundation-layout') {
                __classPrivateFieldSet(this, __container, __classPrivateFieldGet(this, _instances, "m", __tryFindContainingLayout).call(this, this), "f");
            }
        }
    },
    __container = new WeakMap(),
    _instances = new WeakSet(),
    __tryFindContainingLayout = function __tryFindContainingLayout(e) {
        /**
         * This is a special case where the element is part of the layout cache
         * signified by a symbol put on the document cache root
         */
        if (e.getRootNode()[layoutCacheDocument] === true) {
            return { _key: 'foundation-layout-cache' };
        }
        /**
         * If this happens we have found the root of the document
         * without finding a containing FoundationLayout
         * hence, this item is not in a custom layout
         */
        if (e.getRootNode() instanceof Document) {
            return { _key: 'dom' };
        }
        const shadowHost = e.getRootNode().host;
        if (shadowHost.constructor.name === 'FoundationLayout') {
            const layoutHost = shadowHost;
            layoutHost._key = 'foundation-layout';
            return layoutHost;
        }
        return __classPrivateFieldGet(this, _instances, "m", __tryFindContainingLayout).call(this, shadowHost);
    },
    _a; };
/**
 * Stored on the layout's internal cache to signify that the document is not part of the DOM
 * @public
 */
export const layoutCacheDocument = Symbol('layout-cache-document');
