import { DI } from '@microsoft/fast-foundation';
class DefaultJSONSerializer {
    constructor() {
        /**
         * Serializes an object to a JSON string using `JSON.stringify`.
         * @param object - The object to serialize.
         * @returns The serialized JSON string.
         */
        this.serialize = (object) => JSON.stringify(object);
    }
    /**
     * Deserializes a response object from a HTTP request to a JavaScript object using `JSON.parse`.
     * @param input - The HTTP response object.
     * @returns A promise that resolves to the deserialized JavaScript object.
     */
    deserialize(input) {
        if (input && input.ok) {
            return input.text().then((text) => JSON.parse(text));
        }
        if (input && input.data) {
            input = input.data;
        }
        return JSON.parse(input);
    }
}
/**
 * A DI token for the JSON serializer.
 * @public
 */
export const JSONSerializer = DI.createInterface((x) => x.singleton(DefaultJSONSerializer));
